#include "C3DModel.h"
#include <assert.h>


bool C3DModel::NewObject()
{
	if (mStillOpen == true)
		return false;			// Previous session is still incomplete!
	else mStillOpen = true;

	if(mObjectsCount == NoObjects)
		mObjectsCount = 0;
	else mObjectsCount++;

	C3DObject	*objectPtr;
	objectPtr = new C3DObject();

	mObjects.push_back(objectPtr);

	return true;
}

bool C3DModel::EndObject()
{
	if (mStillOpen == false)
		return false;			// No active session
	else mStillOpen = false;
	return true;
}

bool C3DModel::AddVertex(float x, float y, float z)
{
	if (mStillOpen == false)
		return false;			// No active session

	mObjects[mObjectsCount]->vertices.push_back(VECTOR(x, y, z));

	return true;
}

bool C3DModel::AddFace(int A, int B, int C)
{
	if (mStillOpen == false)
		return false;			// No active session

	mObjects[mObjectsCount]->faces.push_back(C3DFace(A, B, C));

	return true;
}

bool C3DModel::AddTexturePoint(float u, float v)
{
	if (mStillOpen == false)
		return false;			// No active session

	mObjects[mObjectsCount]->texturePoints.push_back(VECTOR(u, v, 1));
	return true;
}

bool C3DModel::AddTriMesh()
{
	if(IsObjectWriting() == true)
		EndObject();
	NewObject();

	mObjects[mObjectsCount]->m_sObjName = recName;
	return true;
}

bool C3DModel::AddCamera(C3DCamera Camera)
{
	mCameras.push_back(Camera);
	return true;
}

bool C3DModel::AddLight(float x, float y, float z)
{
	mLights.push_back(C3DLight());
	mLights[ GetLightNumber()-1 ].position = VECTOR(x,y,z);
	mLights[ GetLightNumber()-1 ].m_sLightName = recName;
	return true;
}

bool C3DModel::AddMaterial()
{
	mMaterials.push_back(C3DMaterial());
	return true;
}

bool C3DModel::AddTrackInfo()
{
	mMeshAnim.push_back(C3DTrackInfo());
	return true;
}
bool C3DModel::AddTrackCamera()
{
	mCamAnim.push_back(C3DTrackCamera());
	return true;
}

bool C3DModel::AddTrackOmni()
{
	mOmniAnim.push_back(C3DTrackOmni());
	return true;
}

bool C3DModel::AddTrackSpot()
{
	mSpotAnim.push_back(C3DTrackSpot());
	return true;
}

bool C3DModel::AddTrackCamTarget()
{
	mCamTargetAnim.push_back(C3DTrackCamTarget());
	return true;
}

bool C3DModel::AddTrackSpotTarget()
{
	mSpotTargetAnim.push_back(C3DTrackSpotTarget());
	return true;
}


bool C3DModel::AddTrackPos(const keyf3 &key)
{
	switch( GetCurMode() )
	{
		case TRACKINFO_PARSE:
			mMeshAnim[ GetNumMeshAnims()-1 ].pos.push_back( key );
			break;
		case TRACKCAMERA_PARSE:
			mCamAnim[ GetNumCameraAnims()-1 ].pos.push_back( key );
			break;
		case TRACKLIGHT_PARSE:
			mOmniAnim[ GetNumOmniAnims()-1 ].pos.push_back( key );
			break;
		case TRACKSPOTL_PARSE:
			mSpotAnim[ GetNumSpotAnims()-1 ].pos.push_back( key );
			break;
		case TRACKCAMTGT_PARSE:
			mCamTargetAnim[ GetNumCameraTargetAnims()-1 ].pos.push_back( key );
			break;
		case TRACKLIGHTTGT_PARSE:
			mSpotTargetAnim[ GetNumSpotTargetAnims()-1 ].pos.push_back( key );
			break;
		default:
			assert(false);	// unknown track type.
	}

	return true;
}

bool C3DModel::AddTrackRot(const keyf4 &key)
{
	switch( GetCurMode() )
	{
		case TRACKINFO_PARSE:
			mMeshAnim[ GetNumMeshAnims()-1 ].rot.push_back( key );
			break;
		case TRACKCAMERA_PARSE:
			mCamAnim[ GetNumCameraAnims()-1 ].rot.push_back( key );
			break;
		case TRACKLIGHT_PARSE:
			mOmniAnim[ GetNumOmniAnims()-1 ].rot.push_back( key );
			break;
		case TRACKSPOTL_PARSE:
			mSpotAnim[ GetNumSpotAnims()-1 ].rot.push_back( key );
			break;
		case TRACKCAMTGT_PARSE:
			mCamTargetAnim[ GetNumCameraTargetAnims()-1 ].rot.push_back( key );
			break;
		case TRACKLIGHTTGT_PARSE:
			mSpotTargetAnim[ GetNumSpotTargetAnims()-1 ].rot.push_back( key );
			break;
		default:
			assert(false);	// unknown track type.
	}

	return true;
}

bool C3DModel::AddTrackScale(const keyf3 &key)
{
	switch( GetCurMode() )
	{
		case TRACKINFO_PARSE:
			mMeshAnim[ GetNumMeshAnims()-1 ].scale.push_back( key );
			break;
		case TRACKCAMERA_PARSE:
			mCamAnim[ GetNumCameraAnims()-1 ].scale.push_back( key );
			break;
		case TRACKLIGHT_PARSE:
			mOmniAnim[ GetNumOmniAnims()-1 ].scale.push_back( key );
			break;
		case TRACKSPOTL_PARSE:
			mSpotAnim[ GetNumSpotAnims()-1 ].scale.push_back( key );
			break;
		case TRACKCAMTGT_PARSE:
			mCamTargetAnim[ GetNumCameraTargetAnims()-1 ].scale.push_back( key );
			break;
		case TRACKLIGHTTGT_PARSE:
			mSpotTargetAnim[ GetNumSpotTargetAnims()-1 ].scale.push_back( key );
			break;
		default:
			assert(false);	// unknown track type.
	}

	return true;
}

bool C3DModel::AddTrackFov(const keyf1 &key)
{
	assert( GetCurMode()==TRACKCAMERA_PARSE );
	mCamAnim[ GetNumCameraAnims()-1 ].fov.push_back( key );
	return true;
}

bool C3DModel::AddTrackRoll(const keyf1 &key)
{
	switch( GetCurMode() )
	{
		case TRACKCAMERA_PARSE:
			mCamAnim[ GetNumCameraAnims()-1 ].roll.push_back( key );
			break;
		case TRACKSPOTL_PARSE:
			mSpotAnim[ GetNumSpotAnims()-1 ].roll.push_back( key );
			break;
		default:
			assert(false);	// unknown track type.
	}

	return true;
}

bool C3DModel::AddTrackHotSpot(const keyf1 &key)
{
	assert( GetCurMode()==TRACKSPOTL_PARSE );
	mSpotAnim[ GetNumSpotAnims()-1 ].hotspot.push_back( key );
	return true;
}

bool C3DModel::AddTrackFallOff(const keyf1 &key)
{
	assert( GetCurMode()==TRACKSPOTL_PARSE );
	mSpotAnim[ GetNumSpotAnims()-1 ].falloff.push_back( key );
	return true;
}


bool	C3DModel::AddFaceDescriptor(string Name, int Number)
{
	mObjects[mObjectsCount]->descriptors.push_back(CDescriptor(Name, Number));
	return true;
}

bool C3DModel::AddFaceSmoothRef(unsigned int smoothRef)
{
	mObjects[mObjectsCount]->smoothrefs.push_back(smoothRef);	
	return true;
}

bool C3DModel::SetMatrix(MATRIX Mtx)
{
	if (mStillOpen == false)
		return false;			// No active session

	mObjects[mObjectsCount]->transform = Mtx;
	return true;
}

bool C3DModel::SetTranslation(float x, float y, float z)
{
	if (mStillOpen == false)
		return false;			// No active session

	mObjects[mObjectsCount]->translate[0] = x;
	mObjects[mObjectsCount]->translate[1] = y;
	mObjects[mObjectsCount]->translate[2] = z;
	return true;
}

void C3DModel::UserRGBF(float r, float g, float b)
{
	PARSE_MODE mode = GetCurMode();
	if(mode == LIGHT_PARSE)
	{
		mLights[GetLightNumber()-1].r = r;
		mLights[GetLightNumber()-1].g = g;
		mLights[GetLightNumber()-1].b = b;
	}
}

void C3DModel::UserObjName(string Name)	// chunk: 0x4000
{
	// this might be trimesh, light or camera
	// we'll record the name.
	recName = Name;
}

void C3DModel::User3dFace(unsigned short A, unsigned short B, unsigned short C, 
						  unsigned short Flags)
{
	AddFace((int) A, (int) B, (int) C);
}

void C3DModel::User3dVert(float x, float y, float z)
{
	AddVertex(x, y, z);
}

void C3DModel::UserMapVertex(float U, float V)
{
	AddTexturePoint(U, V);
}

void C3DModel::UserTriMesh()
{
	AddTriMesh();
}

void C3DModel::UserCamera(float px, float py, float pz, float tx, float ty, float tz, float Bank, float Lens)
{
	AddCamera(C3DCamera(VECTOR(px, py, pz), VECTOR(tx, ty, tz), Bank, Lens, recName));
}

void C3DModel::UserLight(float x, float y, float z)
{
	AddLight(x, y, z);
}

void C3DModel::UserSpotLight(float x, float y, float z, float HotSpot, float FallOff)
{
	mLights[ GetLightNumber()-1 ].target = VECTOR(x,y,z);
	mLights[ GetLightNumber()-1 ].hotspot = HotSpot;
	mLights[ GetLightNumber()-1 ].falloff = FallOff;
	mLights[ GetLightNumber()-1 ].is_spot = true;
}

void C3DModel::UserSetSpotRectangular()
{
	mLights[ GetLightNumber()-1 ].is_rectangular = true;
}

void C3DModel::UserSpotRoll(float roll)
{
	mLights[ GetLightNumber()-1 ].roll = roll;
}

void C3DModel::UserSpotAspect(float aspect)
{
	mLights[ GetLightNumber()-1 ].aspect = aspect;
}

void C3DModel::UserInnerRange(float inner)
{
	mLights[ GetLightNumber()-1 ].Near = inner;
}

void C3DModel::UserOuterRange(float outer)
{
	mLights[ GetLightNumber()-1 ].Far = outer;
}


void C3DModel::UserMaterial()
{
	AddMaterial();
}

void C3DModel::UserFaceMaterial(string Name, int Number)
{
	AddFaceDescriptor(Name, Number);
}

void C3DModel::UserFaceSmoothGroup(unsigned int smoothRef)
{
	AddFaceSmoothRef(smoothRef);
}

void C3DModel::UserChunkObj()	// chunk: 0x3d3d
{
	// this chunk appears to have no effect
}

void C3DModel::UserTransformMatrix(const Transform3dsMatrix &Transform, 
								   const Translate3dsMatrix &Translate)
{
	SetTranslation(Translate._11, Translate._12, Translate._13);
	VECTOR	c1(Transform._11, Transform._21, Transform._31);
	VECTOR	c2(Transform._12, Transform._22, Transform._32);
	VECTOR	c3(Transform._13, Transform._23, Transform._33);
	MATRIX	trans(c1, c2, c3);
	SetMatrix(trans);
}

void C3DModel::UserMatName(const string Name)
{
	assert(mMaterials.size()>0);
	mMaterials[mMaterials.size()-1].m_sMatName = Name;
}

void C3DModel::UserMapFile(const string FileName)
{
	assert(mMaterials.size()>0);
	PARSE_MODE mode = GetCurMode();
	if(mode == DIFFUSE_PARSE)
		mMaterials[mMaterials.size()-1].m_sDiffMapName = FileName;
	else if(mode == SPECULAR_PARSE)
		mMaterials[mMaterials.size()-1].m_sSpecMapName = FileName;
	else if(mode == REFLECTION_PARSE)
		mMaterials[mMaterials.size()-1].m_sReflMapName = FileName;
	else if(mode == BUMPMAP_PARSE)
		mMaterials[mMaterials.size()-1].m_sBumpMapName = FileName;
}

void C3DModel::UserFrames(long Start, long End)
{
	frame_begin = Start;
	frame_end = End;
}

void C3DModel::UserTrackInfo()
{
	assert(GetCurMode() == TRACKINFO_PARSE);
	AddTrackInfo();
}

void C3DModel::UserTrackCamera()
{
	assert(GetCurMode() == TRACKCAMERA_PARSE);
	AddTrackCamera();
}

void C3DModel::UserTrackCamTgt()
{
	assert(GetCurMode() == TRACKCAMTGT_PARSE);
	AddTrackCamTarget();
}

void C3DModel::UserTrackLight()
{
	assert(GetCurMode() == TRACKLIGHT_PARSE);
	AddTrackOmni();
}

void C3DModel::UserTrackLightTgt()
{
	assert(GetCurMode() == TRACKLIGHTTGT_PARSE);
	AddTrackSpotTarget();
}

void C3DModel::UserTrackSpotL()
{
	assert(GetCurMode() == TRACKSPOTL_PARSE);
	AddTrackSpot();
}

void C3DModel::UserTrackObjectName(const string Name, int Key1, int Key2, int Key3)
{	
	switch( GetCurMode() )
	{
		case TRACKINFO_PARSE:
			assert(GetNumMeshAnims()>0);
			mMeshAnim[ GetNumMeshAnims()-1 ].m_sIDName = Name;
			mMeshAnim[ GetNumMeshAnims()-1 ].hierarchy_num = Key3;
			break;
		case TRACKCAMERA_PARSE:
			assert(GetNumCameraAnims()>0);
			mCamAnim[ GetNumCameraAnims()-1 ].m_sIDName = Name;
			mCamAnim[ GetNumCameraAnims()-1 ].hierarchy_num = Key3;
			break;
		case TRACKLIGHT_PARSE:
			assert(GetNumOmniAnims()>0);
			mOmniAnim[ GetNumOmniAnims()-1 ].m_sIDName = Name;
			mOmniAnim[ GetNumOmniAnims()-1 ].hierarchy_num = Key3;
			break;
		case TRACKSPOTL_PARSE:
			assert(GetNumSpotAnims()>0);
			mSpotAnim[ GetNumSpotAnims()-1 ].m_sIDName = Name;
			mSpotAnim[ GetNumSpotAnims()-1 ].hierarchy_num = Key3;
			break;
		case TRACKCAMTGT_PARSE:
			assert(GetNumCameraTargetAnims()>0);
			mCamTargetAnim[ GetNumCameraTargetAnims()-1 ].m_sIDName = Name;
			mCamTargetAnim[ GetNumCameraTargetAnims()-1 ].hierarchy_num = Key3;
			break;
		case TRACKLIGHTTGT_PARSE:
			assert(GetNumSpotTargetAnims()>0);
			mSpotTargetAnim[ GetNumSpotTargetAnims()-1 ].m_sIDName = Name;
			mSpotTargetAnim[ GetNumSpotTargetAnims()-1 ].hierarchy_num = Key3;
			break;
		default:
			assert(false);	// unknown track type.
	}
}

void C3DModel::UserTrackInstName( const string trackInstName )
{
	switch( GetCurMode() )
	{
		case TRACKINFO_PARSE:
			assert(GetNumMeshAnims()>0);
			mMeshAnim[ GetNumMeshAnims()-1 ].m_sTrackInstName = trackInstName;
			break;
		case TRACKCAMERA_PARSE:
			assert(GetNumCameraAnims()>0);
			mCamAnim[ GetNumCameraAnims()-1 ].m_sTrackInstName = trackInstName;
			break;
		case TRACKLIGHT_PARSE:
			assert(GetNumOmniAnims()>0);
			mOmniAnim[ GetNumOmniAnims()-1 ].m_sTrackInstName = trackInstName;
			break;
		case TRACKSPOTL_PARSE:
			assert(GetNumSpotAnims()>0);
			mSpotAnim[ GetNumSpotAnims()-1 ].m_sTrackInstName = trackInstName;
			break;
		case TRACKCAMTGT_PARSE:
			assert(GetNumCameraTargetAnims()>0);
			break;
		case TRACKLIGHTTGT_PARSE:
			assert(GetNumSpotTargetAnims()>0);
			break;
		default:
			assert(false);	// unknown track type.
	}
}


void C3DModel::UserPivotPoint(float PivX, float PivY, float PivZ)
{
	switch( GetCurMode() )
	{
		case TRACKINFO_PARSE:
			mMeshAnim[ GetNumMeshAnims()-1 ].pivot_point = VECTOR(PivX, PivY, PivZ);
			break;
		case TRACKCAMERA_PARSE:
			mCamAnim[ GetNumCameraAnims()-1 ].pivot_point = VECTOR(PivX, PivY, PivZ);
			break;
		case TRACKLIGHT_PARSE:
			mOmniAnim[ GetNumOmniAnims()-1 ].pivot_point = VECTOR(PivX, PivY, PivZ);
			break;
		case TRACKSPOTL_PARSE:
			mSpotAnim[ GetNumSpotAnims()-1 ].pivot_point = VECTOR(PivX, PivY, PivZ);
			break;
		case TRACKCAMTGT_PARSE:
			mCamTargetAnim[ GetNumCameraTargetAnims()-1 ].pivot_point = VECTOR(PivX, PivY, PivZ);
			break;
		case TRACKLIGHTTGT_PARSE:
			mSpotTargetAnim[ GetNumSpotTargetAnims()-1 ].pivot_point = VECTOR(PivX, PivY, PivZ);
			break;
		default:
			assert(false);	// unknown track type.
	}
}

void C3DModel::UserObjNumber(int ObjNumber)
{	
	switch( GetCurMode() )
	{
		case TRACKINFO_PARSE:
			mMeshAnim[ GetNumMeshAnims()-1 ].node_id = ObjNumber;
			break;
		case TRACKCAMERA_PARSE:
			mCamAnim[ GetNumCameraAnims()-1 ].node_id = ObjNumber;
			break;
		case TRACKLIGHT_PARSE:
			mOmniAnim[ GetNumOmniAnims()-1 ].node_id = ObjNumber;
			break;
		case TRACKSPOTL_PARSE:
			mSpotAnim[ GetNumSpotAnims()-1 ].node_id = ObjNumber;
			break;
		case TRACKCAMTGT_PARSE:
			mCamTargetAnim[ GetNumCameraTargetAnims()-1 ].node_id = ObjNumber;
			break;
		case TRACKLIGHTTGT_PARSE:
			mSpotTargetAnim[ GetNumSpotTargetAnims()-1 ].node_id = ObjNumber;
			break;
		default:
			assert(false);	// unknown track type.
	}
}

void C3DModel::UserTrackPos(int Frame, float t, float c, float b, float et, float ef, float x, float y, float z)
{
	AddTrackPos( keyf3(Frame, t, c, b, et, ef, x, y, z) );
}

void C3DModel::UserTrackRot(int Frame, float t, float c, float b, float et, float ef, float x, float y, float z, float w)
{
	AddTrackRot( keyf4(Frame, t, c, b, et, ef, x, y, z, w) );
}

void C3DModel::UserTrackScale(int Frame, float t, float c, float b, float et, float ef, float ScaleX, float ScaleY, float ScaleZ)
{
	AddTrackScale( keyf3(Frame, t, c, b, et, ef, ScaleX, ScaleY, ScaleZ) );
}

void C3DModel::UserTrackFov(int Frame, float t, float c, float b, float et, float ef, float fov)
{
	AddTrackFov( keyf1(Frame, t, c, b, et, ef, fov) );
}

void C3DModel::UserTrackRoll(int Frame, float t, float c, float b, float et, float ef, float roll)
{
	AddTrackRoll( keyf1(Frame, t, c, b, et, ef, roll) );
}

void C3DModel::UserTrackHotSpot(int Frame, float t, float c, float b, float et, float ef, float hotspot)
{
	AddTrackHotSpot( keyf1(Frame, t, c, b, et, ef, hotspot) );
}

void C3DModel::UserTrackFallOff(int Frame, float t, float c, float b, float et, float ef, float falloff)
{
	AddTrackFallOff( keyf1(Frame, t, c, b, et, ef, falloff) );
}

void C3DModel::UserEndOfFile()
{
	if(IsObjectWriting() == true)
		EndObject();
}

void C3DObject::UseTransform()
{
	int vnum = vertices.size();

	for (int i = 0; i < vnum; i++)
	{
		vertices[i] = vertices[i] * transform;
	}
}


const C3DTrackInfo * C3DModel::GetTrackInfo(const string NameID) const
{
	bool bFoundMatch = false;
	int i=0;
	while(i<GetNumMeshAnims() && !bFoundMatch)
		if(mMeshAnim[i].m_sIDName == NameID)
			bFoundMatch = true;
		else
			++i;
	return (bFoundMatch) ? &mMeshAnim[i] : NULL;
}

const C3DTrackCamera * C3DModel::GetTrackCamera(const string NameID) const
{
	bool bFoundMatch = false;
	int i=0;
	while(i<GetNumCameraAnims() && !bFoundMatch)
		if(mCamAnim[i].m_sIDName == NameID)
			bFoundMatch = true;
		else
			++i;
	return (bFoundMatch) ? &mCamAnim[i] : NULL;
}

const C3DTrackOmni * C3DModel::GetTrackOmni(const string NameID) const
{
	bool bFoundMatch = false;
	int i=0;
	while(i<GetNumOmniAnims() && !bFoundMatch)
		if(mOmniAnim[i].m_sIDName == NameID)
			bFoundMatch = true;
		else
			++i;
	return (bFoundMatch) ? &mOmniAnim[i] : NULL;
}

const C3DTrackSpot * C3DModel::GetTrackSpot(const string NameID) const
{
	bool bFoundMatch = false;
	int i=0;
	while(i<GetNumSpotAnims() && !bFoundMatch)
		if(mSpotAnim[i].m_sIDName == NameID)
			bFoundMatch = true;
		else
			++i;
	return (bFoundMatch) ? &mSpotAnim[i] : NULL;
}

const C3DTrackCamTarget * C3DModel::GetTrackCamTarget(const string NameID) const
{
	bool bFoundMatch = false;
	int i=0;
	while(i<GetNumCameraTargetAnims() && !bFoundMatch)
		if(mCamTargetAnim[i].m_sIDName == NameID)
			bFoundMatch = true;
		else
			++i;
	return (bFoundMatch) ? &mCamTargetAnim[i] : NULL;
}

const C3DTrackSpotTarget * C3DModel::GetTrackSpotTarget(const string NameID) const
{
	bool bFoundMatch = false;
	int i=0;
	while(i<GetNumSpotTargetAnims() && !bFoundMatch)
		if(mSpotTargetAnim[i].m_sIDName == NameID)
			bFoundMatch = true;
		else
			++i;
	return (bFoundMatch) ? &mSpotTargetAnim[i] : NULL;
}


C3DObject::~C3DObject()
{
	vertices.clear();
	texturePoints.clear();
	faces.clear();
}

