#ifndef __SCRIPTPARSER_H__
#define __SCRIPTPARSER_H__

#include "exttypes.h"
#include "inputcontainer.h"

#define MAX_TOKEN_STRING_LENGTH 25


class CScriptParser
{
public:
	CScriptParser();
	~CScriptParser();

	bool ParseScript(const char pScriptPath[]);
	const CInputContainer GetInputContainer();

private:	// lexical analysis
	enum ASC_TOKEN
	{
		LESS_THEN=0,
		GREATER_THEN,
		LEFT_CURLY,
		RIGHT_CURLY,
		LEFT_BRACKET,
		RIGHT_BRACKET,
		COMMA,
		COLON,
		STRING,

		INTEGER,
		REAL_VALUE,

		EQUAL,
		PIPE,
		INPUT_FILES,
		INPUT_FILE,
		STRIP_MAXVERTS,
		TEXTURE_FOLDERS,
		TEXTURE_FOLDER,
		OUTPUT_FOLDER,
		TEXDEFAULT,
		TEXLIST,
		TEX,

		PACK_INDEX16,
		PACK_INDEX256,
		PACK_RGBA32,
		PACK_NOMIPMAP,
		PACK_MIPMAP,
		PACK_NODITHER,
		PACK_DITHER,

		// length of enum list
		NUM_OF_ASC_TOKENS,
		UNKNOWN = NUM_OF_ASC_TOKENS
	};

	static char token_strings[][MAX_TOKEN_STRING_LENGTH];
	static int32 szeScript;
	static char * pScript;
	static struct Token
	{
		ASC_TOKEN token_type;

		// string location in scene buffer
		int32 str_len;
		char * cptr;	// string is not terminated
	} token;

	bool match(ASC_TOKEN type);
	bool fetch_token();
	float token_to_real();
	int token_to_integer();
	int char_to_digit(const char ch);
	void RemoveWhiteSpace();

private:
	// Input container
	CInputContainer m_cInputContainer;

private:	// parse functions
	bool PriScriptParse();	
	bool ParsePaths(const ASC_TOKEN type);
	bool ParsePath(const ASC_TOKEN type);
	bool ParseOutputFolder();
	bool ParseStripMaxVerts();
	bool ParseListOfSpecificTexProps();
	bool ParseDefaultTexProps();

	bool ParseTextureProps(const bool bIsDefaultProps, CTextureProps * pTexProps);

private:	
	void PrintError(const char err_msg[]);
	
	static int32 m_LineNumber;
};

#endif
